@@ -17,29 +17,23 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-mad
- * @see_also: lame
- *
- * MP3 audio decoder.
- *
- * <refsect2>
- * <title>Example pipelines</title>
- * |[
- * gst-launch filesrc location=music.mp3 ! mad ! audioconvert ! audioresample ! autoaudiosink
- * ]| Decode the mp3 file and play
- * </refsect2>
- */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <stdlib.h>
 #include <string.h>
 #include "gstmad.h"
 #include <gst/audio/audio.h>
 
+
+/* elementfactory information */
+static const GstElementDetails gst_mad_details =
+GST_ELEMENT_DETAILS ("mad mp3 decoder",
+    "Codec/Decoder/Audio",
+    "Uses mad code to decode mp3 streams",
+    "Wim Taymans <wim@fluendo.com>");
+
+
 enum
 {
   ARG_0,
@@ -75,8 +69,10 @@ GST_STATIC_PAD_TEMPLATE ("sink",
         "channels = (int) [ 1, 2 ]")
     );
 
+static void gst_mad_base_init (gpointer g_class);
+static void gst_mad_class_init (GstMadClass * klass);
+static void gst_mad_init (GstMad * mad, GstMadClass * klass);
 static void gst_mad_dispose (GObject * object);
-static void gst_mad_clear_queues (GstMad * mad);
 
 static void gst_mad_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -95,7 +91,6 @@ static gboolean gst_mad_convert_src (GstPad * pad, GstFormat src_format,
 
 static gboolean gst_mad_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn gst_mad_chain (GstPad * pad, GstBuffer * buffer);
-static GstFlowReturn gst_mad_chain_reverse (GstMad * mad, GstBuffer * buf);
 
 static GstStateChangeReturn gst_mad_change_state (GstElement * element,
     GstStateChange transition);
@@ -103,7 +98,13 @@ static GstStateChangeReturn gst_mad_change_state (GstElement * element,
 static void gst_mad_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mad_get_index (GstElement * element);
 
-GST_BOILERPLATE (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT);
+static void
+_do_init (GType type)
+{
+  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
+}
+
+GST_BOILERPLATE_FULL (GstMad, gst_mad, GstElement, GST_TYPE_ELEMENT, _do_init);
 
 /*
 #define GST_TYPE_MAD_LAYER (gst_mad_layer_get_type())
@@ -171,15 +172,12 @@ gst_mad_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &mad_sink_template_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &mad_src_template_factory);
-  gst_element_class_set_details_simple (element_class, "mad mp3 decoder",
-      "Codec/Decoder/Audio",
-      "Uses mad code to decode mp3 streams", "Wim Taymans <wim@fluendo.com>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mad_sink_template_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mad_src_template_factory));
+  gst_element_class_set_details (element_class, &gst_mad_details);
 }
-
 static void
 gst_mad_class_init (GstMadClass * klass)
 {
@@ -205,10 +203,10 @@ gst_mad_class_init (GstMadClass * klass)
    * default values and ranges need to be selected right */
   g_object_class_install_property (gobject_class, ARG_HALF,
       g_param_spec_boolean ("half", "Half", "Generate PCM at 1/2 sample rate",
-          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
-      g_param_spec_boolean ("ignore-crc", "Ignore CRC", "Ignore CRC errors",
-          TRUE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+      g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
+          TRUE, G_PARAM_READWRITE));
 
   /* register tags */
 #define GST_TAG_LAYER    "layer"
@@ -286,10 +284,6 @@ gst_mad_dispose (GObject * object)
   g_free (mad->tempbuffer);
   mad->tempbuffer = NULL;
 
-  g_list_foreach (mad->pending_events, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (mad->pending_events);
-  mad->pending_events = NULL;
-
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -386,7 +380,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
 
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
-  bytes_per_sample = mad->channels * 4;
+  bytes_per_sample = MAD_NCHANNELS (&mad->frame.header) * 4;
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -398,7 +392,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           break;
         case GST_FORMAT_TIME:
         {
-          gint byterate = bytes_per_sample * mad->rate;
+          gint byterate = bytes_per_sample * mad->frame.header.samplerate;
 
           if (byterate == 0)
             return FALSE;
@@ -416,10 +410,10 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           *dest_value = src_value * bytes_per_sample;
           break;
         case GST_FORMAT_TIME:
-          if (mad->rate == 0)
+          if (mad->frame.header.samplerate == 0)
             return FALSE;
           *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              mad->rate);
+              mad->frame.header.samplerate);
           break;
         default:
           res = FALSE;
@@ -432,7 +426,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           /* fallthrough */
         case GST_FORMAT_DEFAULT:
           *dest_value = gst_util_uint64_scale_int (src_value,
-              scale * mad->rate, GST_SECOND);
+              scale * mad->frame.header.samplerate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -618,9 +612,6 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
-  if (rate < 0.0)
-    return FALSE;
-
   if (format == GST_FORMAT_TIME) {
     gst_segment_set_seek (&mad->segment, rate, format, flags, cur_type,
         cur, stop_type, stop, NULL);
@@ -691,9 +682,6 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   gst_event_parse_seek (event, &rate, &format, &flags,
       &cur_type, &cur, &stop_type, &stop);
 
-  if (rate < 0.0)
-    return FALSE;
-
   if (format != GST_FORMAT_TIME) {
     conv = GST_FORMAT_TIME;
     if (!gst_mad_convert_src (pad, format, cur, &conv, &time_cur))
@@ -714,6 +702,9 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   /* shave off the flush flag, we'll need it later */
   flush = ((flags & GST_SEEK_FLAG_FLUSH) != 0);
 
+  /* assume the worst */
+  res = FALSE;
+
   conv = GST_FORMAT_BYTES;
   if (!gst_mad_convert_sink (pad, GST_FORMAT_TIME, time_cur, &conv, &bytes_cur))
     goto convert_error;
@@ -791,22 +782,23 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
   switch (GST_EVENT_TYPE (event)) {
+      /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-      /* the all-formats seek logic, ref the event, we need it later */
       gst_event_ref (event);
-      if (!(res = gst_pad_push_event (mad->sinkpad, event))) {
+      if (!(res = gst_pad_event_default (pad, event))) {
         if (mad->index)
           res = index_seek (mad, pad, event);
         else
           res = normal_seek (mad, pad, event);
       }
-      gst_event_unref (event);
       break;
+
     default:
-      res = gst_pad_push_event (mad->sinkpad, event);
+      res = FALSE;
       break;
   }
 
+  gst_event_unref (event);
   return res;
 }
 
@@ -854,7 +846,6 @@ gst_mad_set_property (GObject * object, guint prop_id,
       break;
   }
 }
-
 static void
 gst_mad_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
@@ -879,9 +870,9 @@ gst_mad_get_property (GObject * object, guint prop_id,
 static void
 gst_mad_update_info (GstMad * mad)
 {
+  gint abr = mad->vbr_average;
   struct mad_header *header = &mad->frame.header;
   gboolean changed = FALSE;
-  GstTagList *list = NULL;
 
 #define CHECK_HEADER(h1,str)                                    \
 G_STMT_START{                                                   \
@@ -895,6 +886,7 @@ G_STMT_START{                                                   \
   if (mad->new_header) {
     mad->framecount = 1;
     mad->vbr_rate = header->bitrate;
+    abr = 0;
   } else {
     mad->framecount++;
     mad->vbr_rate += header->bitrate;
@@ -904,9 +896,11 @@ G_STMT_START{                                                   \
   CHECK_HEADER (layer, "layer");
   CHECK_HEADER (mode, "mode");
   CHECK_HEADER (emphasis, "emphasis");
+  mad->header.bitrate = header->bitrate;
   mad->new_header = FALSE;
 
   if (changed) {
+    GstTagList *list;
     GEnumValue *mode;
     GEnumValue *emphasis;
 
@@ -929,29 +923,15 @@ G_STMT_START{                                                   \
           GST_TAG_AUDIO_CODEC, str, NULL);
       g_free (str);
     }
+    if (!mad->xing_found) {
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, mad->header.bitrate, NULL);
+    }
+    gst_element_post_message (GST_ELEMENT (mad),
+        gst_message_new_tag (GST_OBJECT (mad), list));
   }
-
-  changed = FALSE;
-  CHECK_HEADER (bitrate, "bitrate");
-  if (!mad->xing_found && changed) {
-    if (!list)
-      list = gst_tag_list_new ();
-    gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
-        GST_TAG_BITRATE, mad->header.bitrate, NULL);
-  }
-  mad->header.bitrate = header->bitrate;
 #undef CHECK_HEADER
 
-  if (list) {
-    gst_element_post_message (GST_ELEMENT (mad),
-        gst_message_new_tag (GST_OBJECT (mad), gst_tag_list_copy (list)));
-
-    if (mad->need_newsegment)
-      mad->pending_events =
-          g_list_append (mad->pending_events, gst_event_new_tag (list));
-    else
-      gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
-  }
 }
 
 static gboolean
@@ -995,8 +975,6 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       break;
     }
     case GST_EVENT_EOS:
-      if (mad->segment.rate < 0.0)
-        gst_mad_chain_reverse (mad, NULL);
       mad->caps_set = FALSE;    /* could be a new stream */
       result = gst_pad_push_event (mad->srcpad, event);
       break;
@@ -1006,19 +984,11 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       mad->tempsize = 0;
       mad_frame_mute (&mad->frame);
       mad_synth_mute (&mad->synth);
-      gst_mad_clear_queues (mad);
-      /* fall-through */
-    case GST_EVENT_FLUSH_START:
       result = gst_pad_event_default (pad, event);
+
       break;
     default:
-      if (mad->restart) {
-        /* Cache all other events if we still have to send a NEWSEGMENT */
-        mad->pending_events = g_list_append (mad->pending_events, event);
-        result = TRUE;
-      } else {
-        result = gst_pad_event_default (pad, event);
-      }
+      result = gst_pad_event_default (pad, event);
       break;
   }
   return result;
@@ -1037,10 +1007,10 @@ gst_mad_check_restart (GstMad * mad)
 }
 
 
-/* The following code has been taken from
+/* The following code has been taken from 
  * rhythmbox/metadata/monkey-media/stream-info-impl/id3-vfs/mp3bitrate.c
  * which took it from xine-lib/src/demuxers/demux_mpgaudio.c
- * This code has been kindly relicensed to LGPL by Thibaut Mattern and
+ * This code has been kindly relicensed to LGPL by Thibaut Mattern and 
  * Bastien Nocera
  */
 #define BE_32(x) GST_READ_UINT32_BE(x)
@@ -1090,7 +1060,7 @@ mpg123_parse_xing_header (struct mad_header *header,
   int i;
   guint8 *ptr = (guint8 *) buf;
   double frame_duration;
-  int xflags, xframes, xbytes;
+  int xflags, xframes, xbytes, xvbr_scale;
   int abr;
   guint8 xtoc[XING_TOC_LENGTH];
   int lsf_bit = !(header->flags & MAD_FLAG_LSF_EXT);
@@ -1160,10 +1130,12 @@ mpg123_parse_xing_header (struct mad_header *header,
       ptr += XING_TOC_LENGTH;
     }
 
+    xvbr_scale = -1;
     if (xflags & XING_VBR_SCALE_FLAG) {
       if (ptr >= (buf + bufsize - 4))
         return 0;
-      lprintf ("xvbr_scale: %d\n", BE_32 (ptr));
+      xvbr_scale = BE_32 (ptr);
+      lprintf ("xvbr_scale: %d\n", xvbr_scale);
     }
 
     /* 1 kbit = 1000 bits ! (and not 1024 bits) */
@@ -1271,111 +1243,6 @@ gst_mad_check_caps_reset (GstMad * mad)
   }
 }
 
-static void
-gst_mad_clear_queues (GstMad * mad)
-{
-  g_list_foreach (mad->queued, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (mad->queued);
-  mad->queued = NULL;
-  g_list_foreach (mad->gather, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (mad->gather);
-  mad->gather = NULL;
-  g_list_foreach (mad->decode, (GFunc) gst_mini_object_unref, NULL);
-  g_list_free (mad->decode);
-  mad->decode = NULL;
-}
-
-static GstFlowReturn
-gst_mad_flush_decode (GstMad * mad)
-{
-  GstFlowReturn res = GST_FLOW_OK;
-  GList *walk;
-
-  walk = mad->decode;
-
-  GST_DEBUG_OBJECT (mad, "flushing buffers to decoder");
-
-  /* clear buffer and decoder state */
-  mad->tempsize = 0;
-  mad_frame_mute (&mad->frame);
-  mad_synth_mute (&mad->synth);
-
-  mad->process = TRUE;
-  while (walk) {
-    GList *next;
-    GstBuffer *buf = GST_BUFFER_CAST (walk->data);
-
-    GST_DEBUG_OBJECT (mad, "decoding buffer %p, ts %" GST_TIME_FORMAT,
-        buf, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
-
-    next = g_list_next (walk);
-    /* decode buffer, resulting data prepended to output queue */
-    gst_buffer_ref (buf);
-    res = gst_mad_chain (mad->sinkpad, buf);
-
-    /* if we generated output, we can discard the buffer, else we
-     * keep it in the queue */
-    if (mad->queued) {
-      GST_DEBUG_OBJECT (mad, "decoded buffer to %p", mad->queued->data);
-      mad->decode = g_list_delete_link (mad->decode, walk);
-      gst_buffer_unref (buf);
-    } else {
-      GST_DEBUG_OBJECT (mad, "buffer did not decode, keeping");
-    }
-    walk = next;
-  }
-  mad->process = FALSE;
-
-  /* now send queued data downstream */
-  while (mad->queued) {
-    GstBuffer *buf = GST_BUFFER_CAST (mad->queued->data);
-
-    GST_DEBUG_OBJECT (mad, "pushing buffer %p of size %u, "
-        "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
-        GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
-    res = gst_pad_push (mad->srcpad, buf);
-
-    mad->queued = g_list_delete_link (mad->queued, mad->queued);
-  }
-
-  return res;
-}
-
-static GstFlowReturn
-gst_mad_chain_reverse (GstMad * mad, GstBuffer * buf)
-{
-  GstFlowReturn result = GST_FLOW_OK;
-
-  /* if we have a discont, move buffers to the decode list */
-  if (!buf || GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT)) {
-    GST_DEBUG_OBJECT (mad, "received discont");
-    while (mad->gather) {
-      GstBuffer *gbuf;
-
-      gbuf = GST_BUFFER_CAST (mad->gather->data);
-      /* remove from the gather list */
-      mad->gather = g_list_delete_link (mad->gather, mad->gather);
-      /* copy to decode queue */
-      mad->decode = g_list_prepend (mad->decode, gbuf);
-    }
-    /* decode stuff in the decode queue */
-    gst_mad_flush_decode (mad);
-  }
-
-  if (G_LIKELY (buf)) {
-    GST_DEBUG_OBJECT (mad, "gathering buffer %p of size %u, "
-        "time %" GST_TIME_FORMAT ", dur %" GST_TIME_FORMAT, buf,
-        GST_BUFFER_SIZE (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-        GST_TIME_ARGS (GST_BUFFER_DURATION (buf)));
-
-    /* add buffer to gather queue */
-    mad->gather = g_list_prepend (mad->gather, buf);
-  }
-
-  return result;
-}
-
 static GstFlowReturn
 gst_mad_chain (GstPad * pad, GstBuffer * buffer)
 {
@@ -1383,7 +1250,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
   guint8 *data;
   glong size, tempsize;
   gboolean new_pts = FALSE;
-  gboolean discont;
   GstClockTime timestamp;
   GstFlowReturn result = GST_FLOW_OK;
 
@@ -1395,19 +1261,8 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     GST_DEBUG ("mad restarted");
   }
 
-  if (mad->segment.rate < 0.0) {
-    if (!mad->process)
-      return gst_mad_chain_reverse (mad, buffer);
-    /* no output discont */
-    discont = FALSE;
-  } else {
-    /* take discont flag */
-    discont = GST_BUFFER_IS_DISCONT (buffer);
-  }
-
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
-  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT " duration:%" GST_TIME_FORMAT,
-      GST_TIME_ARGS (timestamp), GST_TIME_ARGS (GST_BUFFER_DURATION (buffer)));
+  GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
 
   /* handle timestamps */
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
@@ -1441,10 +1296,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     gint tocopy;
     guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
-    if (mad->tempsize == 0 && discont) {
-      mad->discont = TRUE;
-      discont = FALSE;
-    }
     tocopy =
         MIN (MAD_BUFFER_MDLEN, MIN (size,
             MAD_BUFFER_MDLEN * 3 - mad->tempsize));
@@ -1511,9 +1362,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
             GST_LOG ("sync error, flushing unneeded data");
             goto next_no_samples;
           }
-        } else if (mad->stream.error == MAD_ERROR_BADDATAPTR) {
-          /* Flush data */
-          goto next_no_samples;
         }
         /* we are in an error state */
         mad->in_error = TRUE;
@@ -1525,7 +1373,45 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           goto end;
         } else if (mad->stream.error == MAD_ERROR_LOSTSYNC) {
           /* lost sync, force a resync */
+          signed long tagsize;
+
           GST_INFO ("recoverable lost sync error");
+
+          tagsize = id3_tag_query (mad->stream.this_frame,
+              mad->stream.bufend - mad->stream.this_frame);
+
+          if (tagsize > mad->tempsize) {
+            GST_INFO ("mad: got partial id3 tag in buffer, skipping");
+          } else if (tagsize > 0) {
+            struct id3_tag *tag;
+            id3_byte_t const *data;
+
+            GST_INFO ("mad: got ID3 tag size %ld", tagsize);
+
+            data = mad->stream.this_frame;
+
+            /* mad has moved the pointer to the next frame over the start of the
+             * id3 tags, so we need to flush one byte less than the tagsize */
+            mad_stream_skip (&mad->stream, tagsize - 1);
+
+            tag = id3_tag_parse (data, tagsize);
+            if (tag) {
+              GstTagList *list;
+
+              list = gst_mad_id3_to_tag_list (tag);
+              id3_tag_delete (tag);
+              GST_DEBUG ("found tag");
+              gst_element_post_message (GST_ELEMENT (mad),
+                  gst_message_new_tag (GST_OBJECT (mad),
+                      gst_tag_list_copy (list)));
+              if (mad->tags) {
+                gst_tag_list_insert (mad->tags, list, GST_TAG_MERGE_PREPEND);
+              } else {
+                mad->tags = gst_tag_list_copy (list);
+              }
+              gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+            }
+          }
         }
 
         mad_frame_mute (&mad->frame);
@@ -1563,12 +1449,7 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               GST_TAG_BITRATE, bitrate, NULL);
           gst_element_post_message (GST_ELEMENT (mad),
               gst_message_new_tag (GST_OBJECT (mad), gst_tag_list_copy (list)));
-
-          if (mad->need_newsegment)
-            mad->pending_events =
-                g_list_append (mad->pending_events, gst_event_new_tag (list));
-          else
-            gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+          gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
 
           goto next_no_samples;
         }
@@ -1580,8 +1461,9 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
-      if (mad->rate == 0) {
-        g_warning ("mad->rate is 0; timestamps cannot be calculated");
+      if (mad->frame.header.samplerate == 0) {
+        g_warning
+            ("mad->frame.header.samplerate is 0; timestamps cannot be calculated");
       } else {
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
@@ -1598,14 +1480,14 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
 
           /* We are using the incoming timestamps to generate the outgoing ones
            * if available. However some muxing formats are not precise enough
-           * to allow us to generate a perfect stream. When converting the
+           * to allow us to generate a perfect stream. When converting the 
            * timestamp to a number of encoded samples so far we are introducing
-           * a lot of potential error compared to our accumulated number of
-           * samples encoded. If the difference between those 2 numbers is
-           * bigger than half a frame we then use the incoming timestamp
+           * a lot of potential error compared to our accumulated number of 
+           * samples encoded. If the difference between those 2 numbers is 
+           * bigger than half a frame we then use the incoming timestamp 
            * as a reference, otherwise we continue using our accumulated samples
            * counter */
-          if (ABS (((gint64) (mad->total_samples)) - total) > nsamples / 2) {
+          if (ABS (mad->total_samples - total) > nsamples / 2) {
             GST_DEBUG_OBJECT (mad, "difference is bigger than half a frame, "
                 "using calculated samples offset %" G_GUINT64_FORMAT, total);
             /* Override our accumulated samples counter */
@@ -1631,8 +1513,7 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
       if (mad->index) {
         guint64 x_bytes = mad->base_byte_offset + mad->bytes_consumed;
 
-        gst_index_add_association (mad->index, mad->index_id,
-            GST_ASSOCIATION_FLAG_DELTA_UNIT,
+        gst_index_add_association (mad->index, mad->index_id, 0,
             GST_FORMAT_BYTES, x_bytes, GST_FORMAT_TIME, time_offset, NULL);
       }
 
@@ -1661,16 +1542,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           mad->need_newsegment = FALSE;
         }
 
-        if (mad->pending_events) {
-          GList *l;
-
-          for (l = mad->pending_events; l != NULL; l = l->next) {
-            gst_pad_push_event (mad->srcpad, GST_EVENT (l->data));
-          }
-          g_list_free (mad->pending_events);
-          mad->pending_events = NULL;
-        }
-
         /* will attach the caps to the buffer */
         result =
             gst_pad_alloc_buffer_and_set_caps (mad->srcpad, 0,
@@ -1683,27 +1554,18 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           goto skip_frame;
         }
 
-        if (GST_BUFFER_SIZE (outbuffer) != nsamples * mad->channels * 4) {
-          gst_buffer_unref (outbuffer);
-
-          outbuffer = gst_buffer_new_and_alloc (nsamples * mad->channels * 4);
-          gst_buffer_set_caps (outbuffer, GST_PAD_CAPS (mad->srcpad));
-        }
-
         mad_synth_frame (&mad->synth, &mad->frame);
         left_ch = mad->synth.pcm.samples[0];
         right_ch = mad->synth.pcm.samples[1];
 
         outdata = (gint32 *) GST_BUFFER_DATA (outbuffer);
 
-        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT " dur: %"
-            GST_TIME_FORMAT, GST_TIME_ARGS (time_offset),
-            GST_TIME_ARGS (time_duration));
+        GST_DEBUG ("mad out timestamp %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (time_offset));
 
         GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
         GST_BUFFER_DURATION (outbuffer) = time_duration;
         GST_BUFFER_OFFSET (outbuffer) = mad->total_samples;
-        GST_BUFFER_OFFSET_END (outbuffer) = mad->total_samples + nsamples;
 
         /* output sample(s) in 16-bit signed native-endian PCM */
         if (mad->channels == 1) {
@@ -1727,21 +1589,8 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
               GST_BUFFER_OFFSET (outbuffer),
               GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
-
-          /* apply discont */
-          if (mad->discont) {
-            GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_DISCONT);
-            mad->discont = FALSE;
-          }
-
           mad->segment.last_stop = GST_BUFFER_TIMESTAMP (outbuffer);
-          if (mad->segment.rate > 0.0) {
-            result = gst_pad_push (mad->srcpad, outbuffer);
-          } else {
-            GST_LOG_OBJECT (mad, "queued buffer");
-            mad->queued = g_list_prepend (mad->queued, outbuffer);
-            result = GST_FLOW_OK;
-          }
+          result = gst_pad_push (mad->srcpad, outbuffer);
           if (result != GST_FLOW_OK) {
             /* Head for the exit, dropping samples as we go */
             goto_exit = TRUE;
@@ -1763,10 +1612,6 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
         mad->bytes_consumed = 0;
       }
       tempsize = 0;
-      if (discont) {
-        mad->discont = TRUE;
-        discont = FALSE;
-      }
 
       if (gst_mad_check_restart (mad)) {
         goto end;
@@ -1818,7 +1663,6 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
       mad_frame_init (&mad->frame);
       mad_synth_init (&mad->synth);
       mad->tempsize = 0;
-      mad->discont = TRUE;
       mad->total_samples = 0;
       mad->rate = 0;
       mad->channels = 0;
@@ -1860,7 +1704,6 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
         gst_tag_list_free (mad->tags);
         mad->tags = NULL;
       }
-      gst_mad_clear_queues (mad);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
@@ -1869,18 +1712,3 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
   }
   return ret;
 }
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
-
-  return gst_element_register (plugin, "mad", GST_RANK_SECONDARY,
-      gst_mad_get_type ());
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "mad",
-    "mp3 decoding based on the mad library",
-    plugin_init, VERSION, "GPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
